//  $Archive:: /SafeTP/base64.h                                           $
//     $Date: 1999/07/14 13:16:38 $
// $Revision: 1.4 $
// Description: Base64 low-level tranformation header (conforms to RFC 2228)
// Copyright 1998, Dan Bonachea  Terms of use are as specified in license.txt.

#ifndef _BASE64_H
#define _BASE64_H

#define BASE64_ERROR  -1

// These functions perform base64 encode/decode as specified in RFC2228
// for encode, inputlen must be set to the length of the binary data to be encoded
// decode assumes the input is null-terminated

// output is written to the output buffer (no overrun checking is done)
// and the functions return the number of bytes written, or BASE64_ERROR if an error occurs
// The functions also write a null terminator at the end of the buffer for convenience,
//   but they do NOT include that null terminator in the length count

// The functions will function properly when the buffers are distinct or overlapping, 
//  but if they overlap, then they must exactly coincide (that is, input == output)

int base64enc(const unsigned char* input, unsigned char* output, int inputlen);
int base64dec(const unsigned char* input, unsigned char* output);

#endif // _BASE64_H


